import 'package:flutter/material.dart';
import 'package:flutter_modular/flutter_modular.dart';
import 'package:flutter_snow_base/src/config/flavor_config.dart';
import 'package:flutter_snow_base/src/models/banner_config_model.dart';
import 'package:flutter_snow_base/src/widgets/device_info_dialog/device_info_dialog.dart';

class FlavorBannerWidget extends StatelessWidget {
  final Widget child;
  final BannerConfigModel bannerConfig = BannerConfigModel(
    bannerName: FlavorConfig.name,
    bannerColor: FlavorConfig.color,
  );

  FlavorBannerWidget({@required this.child});

  @override
  Widget build(BuildContext context) {
    if (FlavorConfig.isProduction) return child;
    return Stack(
      children: <Widget>[
        child,
        _buildBanner(context),
      ],
    );
  }

  Widget _buildBanner(BuildContext context) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onLongPress: () => showDialog(
        context: Modular.navigatorKey.currentState.overlay.context,
        builder: (context) => DeviceInfoDialog(),
      ),
      child: Container(
        width: 50,
        height: 50,
        child: CustomPaint(
          painter: BannerPainter(
            message: bannerConfig.bannerName.toUpperCase(),
            textDirection: Directionality.of(context),
            layoutDirection: Directionality.of(context),
            location: BannerLocation.topStart,
            color: bannerConfig.bannerColor,
          ),
        ),
      ),
    );
  }
}
