import 'package:flutter/material.dart';
import 'package:flutter_snow_base/flutter_snow_base.dart';
import 'package:flutter_snow_base/src/widgets/error_snow/error_snow_widget.dart';

class ListViewWidget<T> extends StatelessWidget {
  final bool useSliver;
  final Resource<List<T>> resource;
  final Widget loadingTile;
  final Widget Function(T data) tileMapper;
  final Widget emptyWidget;
  final Future<void> Function() refresh;
  final EdgeInsets padding;
  final List<Widget> topWidgets;
  final Widget Function(AppException e) errorWidget;

  const ListViewWidget({
    Key key,
    this.useSliver = false,
    @required this.resource,
    @required this.loadingTile,
    @required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.padding,
    this.topWidgets,
    this.errorWidget,
  }) : super(key: key);

  const ListViewWidget.sliver({
    Key key,
    this.useSliver = true,
    @required this.resource,
    @required this.loadingTile,
    @required this.tileMapper,
    this.refresh,
    this.emptyWidget,
    this.topWidgets,
    this.errorWidget,
  })  : padding = null,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    var child = useSliver
        ? SliverList(
            delegate: SliverChildListDelegate.fixed(
              _generateList(
                resource,
              ),
            ),
          )
        : ListView(
            padding: padding,
            shrinkWrap: true,
            children: _generateList(
              resource,
            ),
          );
    if (refresh != null && !useSliver) {
      return RefreshIndicator(
        onRefresh: refresh,
        child: child,
      );
    }
    return child;
  }

  List<Widget> _generateList(
    Resource<List<T>> resource,
  ) {
    var listWidgets = <Widget>[];
    if (resource.data != null) {
      listWidgets = resource.data.map<Widget>(tileMapper).toList();
    }
    switch (resource.status) {
      case Status.loading:
        listWidgets.add(loadingTile);
        listWidgets.add(loadingTile);
        break;
      case Status.success:
        if (emptyWidget != null &&
            (resource.data == null || resource.data.isEmpty)) {
          listWidgets.add(emptyWidget);
        }
        break;
      case Status.failed:
        listWidgets.add(
          errorWidget != null
              ? errorWidget(resource.error)
              : ErrorSnowWidget(
                  resource.error.message,
                  onTryAgain: refresh,
                ),
        );
        break;
      default:
        break;
    }
    if (topWidgets != null) listWidgets.insertAll(0, topWidgets);
    return listWidgets;
  }
}
