import 'package:flutter/material.dart';
import 'package:flutter_snow_base/src/widgets/error_snow/error_snow_widget.dart';

import '../../models/app_exception.dart';
import '../../models/status.dart';
import '../../utils/resource.dart';

class ResourceWidget<T> extends StatelessWidget {
  final Resource<T> resource;
  final Widget loadingWidget;
  final Widget Function(AppException e) errorWidget;
  final bool showErrorWidget;
  final Widget Function(T data) doneWidget;
  final Future<void> Function() refresh;

  const ResourceWidget({
    Key key,
    @required this.resource,
    @required this.loadingWidget,
    this.errorWidget,
    @required this.doneWidget,
    this.refresh,
    this.showErrorWidget = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    switch (resource.status) {
      case Status.loading:
        return loadingWidget;
      case Status.success:
        return doneWidget(resource.data);
      case Status.failed:
        return showErrorWidget
            ? errorWidget == null
                ? ErrorSnowWidget(
                    resource.error.message,
                    onTryAgain: refresh,
                  )
                : errorWidget(resource.error)
            : const SizedBox.shrink();
      default:
        return const SizedBox.shrink();
    }
  }
}
